@@ -11,10 +11,14 @@ from config_generator.components.sanitizers.tsan import TAG
 # fmt: off
 COMPILE_MATRIX = [
     ('ubuntu1804', 'clang', None, ['cyrus']),
+    ('ubuntu2004', 'clang', None, ['cyrus']),
 ]
 
 TEST_OPENSSL_MATRIX = [
-    ('ubuntu1804', 'clang', None, 'cyrus', ['auth'], ['server', 'replica', 'sharded'], ['4.0', '4.2', '4.4', '5.0', '6.0', 'latest']),
+    ('ubuntu1804', 'clang', None, 'cyrus', ['auth'], ['server', 'replica', 'sharded'], ['4.0', '4.2', '4.4', '5.0', '6.0']),
+
+    # Test 7.0+ with Ubuntu 20.04+ since MongoDB 7.0 no longer ships binaries for Ubuntu 18.04.
+    ('ubuntu2004', 'clang', None, 'cyrus', ['auth'], ['server', 'replica', 'sharded'], ['7.0', 'latest']),
 ]
 # fmt: on
 # pylint: enable=line-too-long
