@@ -422,31 +422,31 @@ all_variants = [
         ],
         {"CC": "clang"},
     ),
-    Variant(
-        "aws-ubuntu1804",
-        "AWS Tests (Ubuntu 18.04)",
-        "ubuntu1804-small",
-        [
-            "debug-compile-aws",
-            ".test-aws .4.4",
-            ".test-aws .5.0",
-            ".test-aws .6.0",
-        ],
-        {"CC": "clang"},
-    ),
-    # Test 7.0+ with Ubuntu 20.04+ since MongoDB 7.0 no longer ships binaries for Ubuntu 18.04.
+    # Run AWS tests for MongoDB 4.4 and 5.0 on Ubuntu 20.04. AWS setup scripts expect Ubuntu 20.04+. MongoDB 4.4 and 5.0 are not available on 22.04.
     Variant(
         "aws-ubuntu2004",
         "AWS Tests (Ubuntu 20.04)",
         "ubuntu2004-small",
         [
             "debug-compile-aws",
+            ".test-aws .4.4",
+            ".test-aws .5.0",
+        ],
+        {"CC": "clang"},
+    ),
+    Variant(
+        "aws-ubuntu2204",
+        "AWS Tests (Ubuntu 22.04)",
+        "ubuntu2004-small",
+        [
+            "debug-compile-aws",
+            ".test-aws .6.0",
             ".test-aws .7.0",
             ".test-aws .latest",
         ],
         {"CC": "clang"},
     ),
-    Variant("mongohouse", "Mongohouse Test", "ubuntu1804-test", ["debug-compile-sasl-openssl", "test-mongohouse"], {}),
+    Variant("mongohouse", "Mongohouse Test", "ubuntu2204-small", ["debug-compile-sasl-openssl", "test-mongohouse"], {}),
     Variant(
         "ocsp",
         "OCSP tests",
