@@ -16,6 +16,11 @@ echo "Installing libmongocrypt ... begin"
 }
 echo "Installing libmongocrypt ... end"
 
+# Allow reuse of ccache compilation results between different build directories.
+export CCACHE_BASEDIR CCACHE_NOHASHDIR
+CCACHE_BASEDIR="$(pwd)"
+CCACHE_NOHASHDIR=1
+
 echo "Compile test-azurekms ... begin"
 # Disable unnecessary dependencies. test-azurekms is copied to a remote host for testing, which may not have all dependent libraries.
 "${cmake_binary}" \
