@@ -55,13 +55,6 @@ configure_flags_append "-DCMAKE_PREFIX_PATH=$(native-path "${install_dir}")"
 configure_flags_append "-DENABLE_AUTOMATIC_INIT_AND_CLEANUP=OFF"
 configure_flags_append "-DENABLE_MAINTAINER_FLAGS=ON"
 
-if [[ "${RELEASE:-}" == "ON" ]]; then
-  # Build from the release tarball.
-  mkdir build-dir
-  tar xf ../mongoc.tar.gz -C build-dir --strip-components=1
-  cd build-dir
-fi
-
 configure_flags_append_if_not_null C_STD_VERSION "-DCMAKE_C_STANDARD=${C_STD_VERSION:-}"
 configure_flags_append_if_not_null SASL "-DENABLE_SASL=${SASL:-}"
 configure_flags_append_if_not_null SNAPPY "-DENABLE_SNAPPY=${SNAPPY:-}"
