@@ -1,9 +1,7 @@
 #!/usr/bin/env bash
 
 find_cmake_latest() {
-  # shellcheck source=.evergreen/scripts/env-var-utils.sh
-  . "$(dirname "${BASH_SOURCE[0]}")/env-var-utils.sh"
-  . "$(dirname "${BASH_SOURCE[0]}")/use-tools.sh" paths
+  . "$(dirname "${BASH_SOURCE[0]}")/use-tools.sh" paths || return
 
   declare script_dir
   script_dir="$(to_absolute "$(dirname "${BASH_SOURCE[0]}")")" || return
@@ -11,5 +9,5 @@ find_cmake_latest() {
   # shellcheck source=.evergreen/scripts/find-cmake-version.sh
   . "${script_dir}/find-cmake-version.sh" || return
 
-  find_cmake_version 3 25 2
+  find_cmake_version 3 25 3
 }
