@@ -47,6 +47,10 @@ else
   BSON_ONLY_OPTION="-DENABLE_MONGOC=ON"
 fi
 
+# Allow reuse of ccache compilation results between different build directories.
+export CCACHE_BASEDIR CCACHE_NOHASHDIR
+CCACHE_BASEDIR="$SCRATCH_DIR"
+CCACHE_NOHASHDIR=1
 
 $CMAKE -DCMAKE_INSTALL_PREFIX=$INSTALL_PREFIX -DCMAKE_PREFIX_PATH=$INSTALL_DIR/lib/cmake $BSON_ONLY_OPTION "$SCRATCH_DIR"
 $CMAKE --build .
@@ -201,4 +205,3 @@ if test -d $INSTALL_DIR/share/mongo-c-driver; then
 else
   echo "$INSTALL_DIR/share/mongo-c-driver check ok"
 fi
-
