@@ -16,12 +16,17 @@
 
 #include "common-thread-private.h"
 
+#include <errno.h>
+
 #if defined(BSON_OS_UNIX)
 int
 mcommon_thread_create (bson_thread_t *thread,
                        BSON_THREAD_FUN_TYPE (func),
                        void *arg)
 {
+   BSON_ASSERT_PARAM (thread);
+   BSON_ASSERT_PARAM (func);
+   BSON_ASSERT (arg || true); // optional.
    return pthread_create (thread, NULL, func, arg);
 }
 int
@@ -45,9 +50,13 @@ mcommon_thread_create (bson_thread_t *thread,
                        BSON_THREAD_FUN_TYPE (func),
                        void *arg)
 {
+   BSON_ASSERT_PARAM (thread);
+   BSON_ASSERT_PARAM (func);
+   BSON_ASSERT (arg || true); // optional.
+
    *thread = (HANDLE) _beginthreadex (NULL, 0, func, arg, 0, NULL);
    if (0 == *thread) {
-      return 1;
+      return errno;
    }
    return 0;
 }
