@@ -292,13 +292,12 @@ test_bson_oid_init_with_threads (void)
 
       for (i = 0; i < N_THREADS; i++) {
          contexts[i] = bson_context_new (flags);
-         r = COMMON_PREFIX (thread_create) (
-            &threads[i], oid_worker, contexts[i]);
+         r = mcommon_thread_create (&threads[i], oid_worker, contexts[i]);
          BSON_ASSERT (r == 0);
       }
 
       for (i = 0; i < N_THREADS; i++) {
-         COMMON_PREFIX (thread_join) (threads[i]);
+         mcommon_thread_join (threads[i]);
       }
 
       for (i = 0; i < N_THREADS; i++) {
@@ -315,12 +314,12 @@ test_bson_oid_init_with_threads (void)
       context = bson_context_new (BSON_CONTEXT_THREAD_SAFE);
 
       for (i = 0; i < N_THREADS; i++) {
-         r = COMMON_PREFIX (thread_create) (&threads[i], oid_worker, context);
+         r = mcommon_thread_create (&threads[i], oid_worker, context);
          BSON_ASSERT (r == 0);
       }
 
       for (i = 0; i < N_THREADS; i++) {
-         r = COMMON_PREFIX (thread_join) (threads[i]);
+         r = mcommon_thread_join (threads[i]);
          BSON_ASSERT (r == 0);
       }
 
