@@ -54,7 +54,7 @@ print_pipeline (mongoc_collection_t *collection)
 }
 
 int
-main (int argc, char *argv[])
+main (void)
 {
    mongoc_client_t *client;
    mongoc_collection_t *collection;
