@@ -21,7 +21,7 @@
  * Internal struct to represent a command we will send to the server - command
  * parameters are collected in a mongoc_cmd_parts_t until we know the server's
  * wire version and whether it is mongos, then we collect the parts into a
- * mongoc_cmd_t, and gather that into a mongoc_rpc_t.
+ * mongoc_cmd_t, and gather that into an RPC message.
  */
 
 #ifndef MONGOC_CMD_PRIVATE_H
@@ -65,6 +65,7 @@ typedef struct _mongoc_cmd_t {
    mongoc_server_api_t *api;
    bool is_acknowledged;
    bool is_txn_finish;
+   bool op_msg_is_exhaust;
 } mongoc_cmd_t;
 
 
@@ -107,25 +108,21 @@ mongoc_cmd_parts_set_server_api (mongoc_cmd_parts_t *parts,
 bool
 mongoc_cmd_parts_append_opts (mongoc_cmd_parts_t *parts,
                               bson_iter_t *iter,
-                              int max_wire_version,
                               bson_error_t *error);
 
 bool
 mongoc_cmd_parts_set_read_concern (mongoc_cmd_parts_t *parts,
                                    const mongoc_read_concern_t *rc,
-                                   int max_wire_version,
                                    bson_error_t *error);
 
 bool
 mongoc_cmd_parts_set_write_concern (mongoc_cmd_parts_t *parts,
                                     const mongoc_write_concern_t *wc,
-                                    int max_wire_version,
                                     bson_error_t *error);
 
 bool
 mongoc_cmd_parts_append_read_write (mongoc_cmd_parts_t *parts,
                                     mongoc_read_write_opts_t *rw_opts,
-                                    int max_wire_version,
                                     bson_error_t *error);
 
 bool
