@@ -33,6 +33,8 @@ mongoc_cmd_parts_init (mongoc_cmd_parts_t *parts,
                        mongoc_query_flags_t user_query_flags,
                        const bson_t *command_body)
 {
+   BSON_ASSERT_PARAM (client);
+
    parts->body = command_body;
    parts->user_query_flags = user_query_flags;
    parts->read_prefs = NULL;
@@ -52,6 +54,7 @@ mongoc_cmd_parts_init (mongoc_cmd_parts_t *parts,
    parts->assembled.db_name = db_name;
    parts->assembled.command = NULL;
    parts->assembled.query_flags = MONGOC_QUERY_NONE;
+   parts->assembled.op_msg_is_exhaust = false;
    parts->assembled.payload_identifier = NULL;
    parts->assembled.payload = NULL;
    parts->assembled.session = NULL;
@@ -108,7 +111,6 @@ mongoc_cmd_parts_set_session (mongoc_cmd_parts_t *parts,
 bool
 mongoc_cmd_parts_append_opts (mongoc_cmd_parts_t *parts,
                               bson_iter_t *iter,
-                              int max_wire_version,
                               bson_error_t *error)
 {
    mongoc_client_session_t *cs = NULL;
@@ -124,23 +126,13 @@ mongoc_cmd_parts_append_opts (mongoc_cmd_parts_t *parts,
    BSON_ASSERT (!parts->assembled.command);
 
    while (bson_iter_next (iter)) {
-      if (BSON_ITER_IS_KEY (iter, "collation")) {
-         if (max_wire_version < WIRE_VERSION_COLLATION) {
-            bson_set_error (error,
-                            MONGOC_ERROR_COMMAND,
-                            MONGOC_ERROR_PROTOCOL_BAD_WIRE_VERSION,
-                            "The selected server does not support collation");
-            RETURN (false);
-         }
-
-      } else if (BSON_ITER_IS_KEY (iter, "writeConcern")) {
+      if (BSON_ITER_IS_KEY (iter, "writeConcern")) {
          wc = _mongoc_write_concern_new_from_iter (iter, error);
          if (!wc) {
             RETURN (false);
          }
 
-         if (!mongoc_cmd_parts_set_write_concern (
-                parts, wc, max_wire_version, error)) {
+         if (!mongoc_cmd_parts_set_write_concern (parts, wc, error)) {
             mongoc_write_concern_destroy (wc);
             RETURN (false);
          }
@@ -148,14 +140,6 @@ mongoc_cmd_parts_append_opts (mongoc_cmd_parts_t *parts,
          mongoc_write_concern_destroy (wc);
          continue;
       } else if (BSON_ITER_IS_KEY (iter, "readConcern")) {
-         if (max_wire_version < WIRE_VERSION_READ_CONCERN) {
-            bson_set_error (error,
-                            MONGOC_ERROR_COMMAND,
-                            MONGOC_ERROR_PROTOCOL_BAD_WIRE_VERSION,
-                            "The selected server does not support readConcern");
-            RETURN (false);
-         }
-
          if (!BSON_ITER_HOLDS_DOCUMENT (iter)) {
             bson_set_error (error,
                             MONGOC_ERROR_COMMAND,
@@ -213,7 +197,6 @@ mongoc_cmd_parts_append_opts (mongoc_cmd_parts_t *parts,
 bool
 mongoc_cmd_parts_set_read_concern (mongoc_cmd_parts_t *parts,
                                    const mongoc_read_concern_t *rc,
-                                   int max_wire_version,
                                    bson_error_t *error)
 {
    const char *command_name;
@@ -238,18 +221,6 @@ mongoc_cmd_parts_set_read_concern (mongoc_cmd_parts_t *parts,
       RETURN (true);
    }
 
-   if (max_wire_version < WIRE_VERSION_READ_CONCERN) {
-      bson_set_error (error,
-                      MONGOC_ERROR_COMMAND,
-                      MONGOC_ERROR_PROTOCOL_BAD_WIRE_VERSION,
-                      "\"%s\" command does not support readConcern with "
-                      "wire version %d, wire version %d is required",
-                      command_name,
-                      max_wire_version,
-                      WIRE_VERSION_READ_CONCERN);
-      RETURN (false);
-   }
-
    bson_destroy (&parts->read_concern_document);
    bson_copy_to (_mongoc_read_concern_get_bson ((mongoc_read_concern_t *) rc),
                  &parts->read_concern_document);
@@ -263,40 +234,24 @@ mongoc_cmd_parts_set_read_concern (mongoc_cmd_parts_t *parts,
 bool
 mongoc_cmd_parts_set_write_concern (mongoc_cmd_parts_t *parts,
                                     const mongoc_write_concern_t *wc,
-                                    int max_wire_version,
                                     bson_error_t *error)
 {
-   const char *command_name;
-   bool is_fam;
-   bool wc_allowed;
-
    ENTRY;
 
    if (!wc) {
       RETURN (true);
    }
 
-   command_name = _mongoc_get_command_name (parts->body);
+   const char *const command_name = _mongoc_get_command_name (parts->body);
 
    if (!command_name) {
       OPTS_ERR (COMMAND_INVALID_ARG, "Empty command document");
    }
 
-   is_fam = !strcasecmp (command_name, "findandmodify");
-
-   wc_allowed =
-      parts->is_write_command ||
-      (is_fam && max_wire_version >= WIRE_VERSION_FAM_WRITE_CONCERN) ||
-      (!is_fam && max_wire_version >= WIRE_VERSION_CMD_WRITE_CONCERN);
-
-   if (wc_allowed) {
-      parts->assembled.is_acknowledged =
-         mongoc_write_concern_is_acknowledged (wc);
-      bson_destroy (&parts->write_concern_document);
-      bson_copy_to (
-         _mongoc_write_concern_get_bson ((mongoc_write_concern_t *) wc),
-         &parts->write_concern_document);
-   }
+   parts->assembled.is_acknowledged = mongoc_write_concern_is_acknowledged (wc);
+   bson_destroy (&parts->write_concern_document);
+   bson_copy_to (_mongoc_write_concern_get_bson ((mongoc_write_concern_t *) wc),
+                 &parts->write_concern_document);
 
    RETURN (true);
 }
@@ -324,7 +279,6 @@ mongoc_cmd_parts_set_write_concern (mongoc_cmd_parts_t *parts,
 bool
 mongoc_cmd_parts_append_read_write (mongoc_cmd_parts_t *parts,
                                     mongoc_read_write_opts_t *rw_opts,
-                                    int max_wire_version,
                                     bson_error_t *error)
 {
    ENTRY;
@@ -333,11 +287,6 @@ mongoc_cmd_parts_append_read_write (mongoc_cmd_parts_t *parts,
    BSON_ASSERT (!parts->assembled.command);
 
    if (!bson_empty (&rw_opts->collation)) {
-      if (max_wire_version < WIRE_VERSION_COLLATION) {
-         OPTS_ERR (PROTOCOL_BAD_WIRE_VERSION,
-                   "The selected server does not support collation");
-      }
-
       if (!bson_append_document (
              &parts->extra, "collation", 9, &rw_opts->collation)) {
          OPTS_ERR (COMMAND_INVALID_ARG, "'opts' with 'collation' is too large");
@@ -345,17 +294,12 @@ mongoc_cmd_parts_append_read_write (mongoc_cmd_parts_t *parts,
    }
 
    if (!mongoc_cmd_parts_set_write_concern (
-          parts, rw_opts->writeConcern, max_wire_version, error)) {
+          parts, rw_opts->writeConcern, error)) {
       RETURN (false);
    }
 
    /* process explicit read concern */
    if (!bson_empty (&rw_opts->readConcern)) {
-      if (max_wire_version < WIRE_VERSION_READ_CONCERN) {
-         OPTS_ERR (PROTOCOL_BAD_WIRE_VERSION,
-                   "The selected server does not support readConcern");
-      }
-
       /* save readConcern for later, once we know about causal consistency */
       bson_destroy (&parts->read_concern_document);
       bson_copy_to (&rw_opts->readConcern, &parts->read_concern_document);
@@ -707,7 +651,7 @@ _allow_txn_number (const mongoc_cmd_parts_t *parts,
       return false;
    }
 
-   if (server_stream->sd->max_wire_version < WIRE_VERSION_RETRY_WRITES) {
+   if (server_stream->retry_attempted) {
       return false;
    }
 
@@ -740,7 +684,7 @@ _is_retryable_write (const mongoc_cmd_parts_t *parts,
       return false;
    }
 
-   if (server_stream->sd->max_wire_version < WIRE_VERSION_RETRY_WRITES) {
+   if (server_stream->retry_attempted) {
       return false;
    }
 
@@ -777,7 +721,7 @@ _is_retryable_read (const mongoc_cmd_parts_t *parts,
       return false;
    }
 
-   if (server_stream->sd->max_wire_version < WIRE_VERSION_RETRY_READS) {
+   if (server_stream->retry_attempted) {
       return false;
    }
 
@@ -827,7 +771,7 @@ mongoc_cmd_parts_assemble (mongoc_cmd_parts_t *parts,
    const char *cmd_name;
    bool is_get_more;
    const mongoc_read_prefs_t *prefs_ptr;
-   mongoc_read_mode_t mode = mongoc_read_prefs_get_mode (parts->read_prefs);
+   mongoc_read_mode_t mode;
    bool ret = false;
 
    ENTRY;
@@ -836,6 +780,7 @@ mongoc_cmd_parts_assemble (mongoc_cmd_parts_t *parts,
    BSON_ASSERT (server_stream);
 
    server_type = server_stream->sd->type;
+
    cs = parts->prohibit_lsid ? NULL : parts->assembled.session;
 
    /* Assembling the command depends on the type of server. If the server has
@@ -894,7 +839,9 @@ mongoc_cmd_parts_assemble (mongoc_cmd_parts_t *parts,
       mode = MONGOC_READ_PRIMARY;
    }
 
-   if (server_stream->sd->max_wire_version >= WIRE_VERSION_OP_MSG) {
+   if (mongoc_client_uses_server_api (parts->client) ||
+       mongoc_client_uses_loadbalanced (parts->client) ||
+       server_stream->sd->max_wire_version >= WIRE_VERSION_MIN) {
       if (!bson_has_field (parts->body, "$db")) {
          BSON_APPEND_UTF8 (&parts->extra, "$db", parts->assembled.db_name);
       }
@@ -992,7 +939,7 @@ mongoc_cmd_parts_assemble (mongoc_cmd_parts_t *parts,
       }
 
       /* Add versioned server api, if it is set. */
-      if (parts->client->api) {
+      if (mongoc_client_uses_server_api (parts->client)) {
          _mongoc_cmd_append_server_api (&parts->assembled_body,
                                         parts->client->api);
       }
@@ -1127,10 +1074,7 @@ _mongoc_cmd_append_payload_as_array (const mongoc_cmd_t *cmd, bson_t *out)
    bson_t doc;
    const uint8_t *pos;
    const char *field_name;
-   bson_t bson;
-   char str[16];
-   const char *key;
-   uint32_t i;
+   bson_array_builder_t *bson;
 
    BSON_ASSERT (cmd->payload && cmd->payload_size);
 
@@ -1138,22 +1082,19 @@ _mongoc_cmd_append_payload_as_array (const mongoc_cmd_t *cmd, bson_t *out)
     * "update", or "delete" command. */
    field_name = _mongoc_get_documents_field_name (cmd->command_name);
    BSON_ASSERT (field_name);
-   BSON_ASSERT (BSON_APPEND_ARRAY_BEGIN (out, field_name, &bson));
+   BSON_ASSERT (BSON_APPEND_ARRAY_BUILDER_BEGIN (out, field_name, &bson));
 
    pos = cmd->payload;
-   i = 0;
    while (pos < cmd->payload + cmd->payload_size) {
       memcpy (&doc_len, pos, sizeof (doc_len));
       doc_len = BSON_UINT32_FROM_LE (doc_len);
       BSON_ASSERT (bson_init_static (&doc, pos, (size_t) doc_len));
-      bson_uint32_to_string (i, &key, str, sizeof (str));
-      BSON_APPEND_DOCUMENT (&bson, key, &doc);
+      bson_array_builder_append_document (bson, &doc);
 
       pos += doc_len;
-      i++;
    }
 
-   bson_append_array_end (out, &bson);
+   bson_append_array_builder_end (out, bson);
 }
 
 /*--------------------------------------------------------------------------
@@ -1182,6 +1123,8 @@ _mongoc_cmd_append_server_api (bson_t *command_body,
 
    string_version = mongoc_server_api_version_to_string (api->version);
 
+   BSON_ASSERT (string_version);
+
    bson_append_utf8 (command_body, "apiVersion", -1, string_version, -1);
 
    if (api->strict.is_set) {
