@@ -1,5 +1,5 @@
 /*
- * Copyright 2016 MongoDB, Inc.
+ * Copyright 2016-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -36,6 +36,8 @@
 #include "mongoc-version.h"
 #include "mongoc-util-private.h"
 
+#include <bson-dsl.h>
+
 /*
  * Global handshake data instance. Initialized at startup from mongoc_init
  *
@@ -61,14 +63,10 @@ _set_bit (uint8_t *bf, uint32_t byte_count, uint32_t bit)
 char *
 _mongoc_handshake_get_config_hex_string (void)
 {
-   uint32_t byte_count;
-   uint8_t *bf;
-   bson_string_t *str;
-   int i;
-
-   byte_count = (LAST_MONGOC_MD_FLAG + 7) / 8; /* ceil (num_bits / 8) */
+   const uint32_t byte_count =
+      (LAST_MONGOC_MD_FLAG + 7) / 8; /* ceil (num_bits / 8) */
    /* allocate enough bytes to fit all config bits. */
-   bf = (uint8_t *) bson_malloc0 (byte_count);
+   uint8_t *const bf = (uint8_t *) bson_malloc0 (byte_count);
 
 #ifdef MONGOC_ENABLE_SSL_SECURE_CHANNEL
    _set_bit (bf, byte_count, MONGOC_ENABLE_SSL_SECURE_CHANNEL);
@@ -182,13 +180,9 @@ _mongoc_handshake_get_config_hex_string (void)
    _set_bit (bf, byte_count, MONGOC_MD_FLAG_ENABLE_SHM_COUNTERS);
 #endif
 
-#ifdef MONGOC_TRACE
-   _set_bit (bf, byte_count, MONGOC_MD_FLAG_TRACE);
-#endif
-
-#ifdef MONGOC_ENABLE_ICU
-   _set_bit (bf, byte_count, MONGOC_MD_FLAG_ENABLE_ICU);
-#endif
+   if (MONGOC_TRACE_ENABLED) {
+      _set_bit (bf, byte_count, MONGOC_MD_FLAG_TRACE);
+   }
 
 #ifdef MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION
    _set_bit (bf, byte_count, MONGOC_MD_FLAG_ENABLE_CLIENT_SIDE_ENCRYPTION);
@@ -198,8 +192,12 @@ _mongoc_handshake_get_config_hex_string (void)
    _set_bit (bf, byte_count, MONGOC_MD_FLAG_ENABLE_MONGODB_AWS_AUTH);
 #endif
 
-   str = bson_string_new ("0x");
-   for (i = 0; i < byte_count; i++) {
+   if (MONGOC_SRV_ENABLED) {
+      _set_bit (bf, byte_count, MONGOC_MD_FLAG_ENABLE_SRV);
+   }
+
+   bson_string_t *const str = bson_string_new ("0x");
+   for (uint32_t i = 0u; i < byte_count; i++) {
       bson_string_append_printf (str, "%02x", bf[i]);
    }
    bson_free (bf);
@@ -295,7 +293,15 @@ _get_os_version (void)
    ZeroMemory (&osvi, sizeof (OSVERSIONINFO));
    osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
 
-   if (GetVersionEx (&osvi)) {
+#if defined(_MSC_VER)
+   // CDRIVER-4263: GetVersionEx is deprecated.
+#pragma warning(suppress : 4996)
+   const BOOL res = GetVersionEx (&osvi);
+#else
+   const BOOL res = GetVersionEx (&osvi);
+#endif
+
+   if (res) {
       bson_snprintf (ret,
                      HANDSHAKE_OS_VERSION_MAX,
                      "%lu.%lu (%lu)",
@@ -378,6 +384,93 @@ _set_platform_string (mongoc_handshake_t *handshake)
    handshake->platform = bson_string_free (str, false);
 }
 
+static void
+_free_env_info (mongoc_handshake_t *handshake)
+{
+   bson_free (handshake->env_region);
+}
+
+static void
+_get_env_info (mongoc_handshake_t *handshake)
+{
+   char *aws_env = _mongoc_getenv ("AWS_EXECUTION_ENV");
+   char *aws_lambda = _mongoc_getenv ("AWS_LAMBDA_RUNTIME_API");
+   char *vercel_env = _mongoc_getenv ("VERCEL");
+   char *azure_env = _mongoc_getenv ("FUNCTIONS_WORKER_RUNTIME");
+   char *gcp_env = _mongoc_getenv ("K_SERVICE");
+   char *memory_str = NULL;
+   char *timeout_str = NULL;
+   char *region_str = NULL;
+
+   bool is_aws = (aws_env && strlen (aws_env) &&
+                  (aws_env == strstr (aws_env, "AWS_Lambda_"))) ||
+                 (aws_lambda && strlen (aws_lambda));
+   bool is_vercel = vercel_env && strlen (vercel_env);
+   bool is_azure = azure_env && strlen (azure_env);
+   bool is_gcp = gcp_env && strlen (gcp_env);
+
+   handshake->env = MONGOC_HANDSHAKE_ENV_NONE;
+   handshake->env_region = NULL;
+   handshake->env_memory_mb.set = false;
+   handshake->env_timeout_sec.set = false;
+
+   if ((is_aws || is_vercel) + is_azure + is_gcp != 1) {
+      goto cleanup;
+   }
+
+   if (is_aws && !is_vercel) {
+      handshake->env = MONGOC_HANDSHAKE_ENV_AWS;
+      region_str = _mongoc_getenv ("AWS_REGION");
+      memory_str = _mongoc_getenv ("AWS_LAMBDA_FUNCTION_MEMORY_SIZE");
+   } else if (is_vercel) {
+      handshake->env = MONGOC_HANDSHAKE_ENV_VERCEL;
+      region_str = _mongoc_getenv ("VERCEL_REGION");
+   } else if (is_gcp) {
+      handshake->env = MONGOC_HANDSHAKE_ENV_GCP;
+      region_str = _mongoc_getenv ("FUNCTION_REGION");
+      memory_str = _mongoc_getenv ("FUNCTION_MEMORY_MB");
+      timeout_str = _mongoc_getenv ("FUNCTION_TIMEOUT_SEC");
+   } else if (is_azure) {
+      handshake->env = MONGOC_HANDSHAKE_ENV_AZURE;
+   }
+
+   if (memory_str) {
+      char *endptr;
+      int64_t env_memory_mb = bson_ascii_strtoll (memory_str, &endptr, 10);
+      bool parse_ok = endptr == memory_str + (strlen (memory_str));
+      bool in_range = bson_in_range_int32_t_signed (env_memory_mb);
+
+      if (parse_ok && in_range) {
+         handshake->env_memory_mb.set = true;
+         handshake->env_memory_mb.value = (int32_t) env_memory_mb;
+      }
+   }
+   if (timeout_str) {
+      char *endptr;
+      int64_t env_timeout_sec = bson_ascii_strtoll (timeout_str, &endptr, 10);
+      bool parse_ok = endptr == timeout_str + (strlen (timeout_str));
+      bool in_range = bson_in_range_int32_t_signed (env_timeout_sec);
+
+      if (parse_ok && in_range) {
+         handshake->env_timeout_sec.set = true;
+         handshake->env_timeout_sec.value = (int32_t) env_timeout_sec;
+      }
+   }
+   if (region_str && strlen (region_str)) {
+      handshake->env_region = bson_strdup (region_str);
+   }
+
+cleanup:
+   bson_free (aws_env);
+   bson_free (aws_lambda);
+   bson_free (vercel_env);
+   bson_free (azure_env);
+   bson_free (gcp_env);
+   bson_free (memory_str);
+   bson_free (timeout_str);
+   bson_free (region_str);
+}
+
 static void
 _set_compiler_info (mongoc_handshake_t *handshake)
 {
@@ -440,6 +533,7 @@ _mongoc_handshake_init (void)
    _get_system_info (_mongoc_handshake_get ());
    _get_driver_info (_mongoc_handshake_get ());
    _set_platform_string (_mongoc_handshake_get ());
+   _get_env_info (_mongoc_handshake_get ());
    _set_compiler_info (_mongoc_handshake_get ());
    _set_flags (_mongoc_handshake_get ());
 
@@ -450,117 +544,205 @@ _mongoc_handshake_init (void)
 void
 _mongoc_handshake_cleanup (void)
 {
-   _free_system_info (_mongoc_handshake_get ());
-   _free_driver_info (_mongoc_handshake_get ());
-   _free_platform_string (_mongoc_handshake_get ());
+   mongoc_handshake_t *h = _mongoc_handshake_get ();
+   _free_system_info (h);
+   _free_driver_info (h);
+   _free_platform_string (h);
+   _free_env_info (h);
+   *h = (mongoc_handshake_t){0};
 
    bson_mutex_destroy (&gHandshakeLock);
 }
 
 static void
-_append_platform_field (bson_t *doc, const char *platform)
+_append_platform_field (bson_t *doc, const char *platform, bool truncate)
 {
-   int max_platform_str_size;
-
    char *compiler_info = _mongoc_handshake_get ()->compiler_info;
    char *flags = _mongoc_handshake_get ()->flags;
    bson_string_t *combined_platform = bson_string_new (platform);
 
    /* Compute space left for platform field */
-   max_platform_str_size =
+   const int max_platform_str_size =
       HANDSHAKE_MAX_SIZE - ((int) doc->len +
                             /* 1 byte for utf8 tag */
                             1 +
 
                             /* key size */
-                            (int) strlen (HANDSHAKE_PLATFORM_FIELD) +
-                            1 +
+                            (int) strlen (HANDSHAKE_PLATFORM_FIELD) + 1 +
 
                             /* 4 bytes for length of string */
                             4);
 
-   if (max_platform_str_size <= 0) {
+   if (truncate && max_platform_str_size <= 0) {
+      bson_string_free (combined_platform, true);
       return;
    }
 
    /* We opt to drop compiler info and flags if they can't fit, while the
     * platform information is truncated
-    * Try to drop flags first, and if there is still not enough space also drop
-    * compiler info */
-   if (max_platform_str_size >
-       combined_platform->len + strlen (compiler_info) + 1) {
+    * Try to drop flags first, and if there is still not enough space also
+    * drop compiler info */
+   if (!truncate || bson_cmp_greater_equal_su (
+                       max_platform_str_size,
+                       combined_platform->len + strlen (compiler_info) + 1u)) {
       bson_string_append (combined_platform, compiler_info);
    }
-   if (max_platform_str_size > combined_platform->len + strlen (flags) + 1) {
+   if (!truncate || bson_cmp_greater_equal_su (max_platform_str_size,
+                                               combined_platform->len +
+                                                  strlen (flags) + 1u)) {
       bson_string_append (combined_platform, flags);
    }
 
    /* We use the flags_index field to check if the CLAGS/LDFLAGS need to be
     * truncated, and if so we drop them altogether */
+   BSON_ASSERT (bson_in_range_unsigned (int, combined_platform->len));
+   int length = truncate ? BSON_MIN (max_platform_str_size - 1,
+                                     (int) combined_platform->len)
+                         : -1;
    bson_append_utf8 (
-      doc,
-      HANDSHAKE_PLATFORM_FIELD,
-      -1,
-      combined_platform->str,
-      BSON_MIN (max_platform_str_size - 1, combined_platform->len));
+      doc, HANDSHAKE_PLATFORM_FIELD, -1, combined_platform->str, length);
 
    bson_string_free (combined_platform, true);
-   BSON_ASSERT (doc->len <= HANDSHAKE_MAX_SIZE);
+}
+
+static bool
+_get_subdoc_static (bson_t *doc, char *subdoc_name, bson_t *out)
+{
+   bson_iter_t iter;
+   if (bson_iter_init_find (&iter, doc, subdoc_name) &&
+       BSON_ITER_HOLDS_DOCUMENT (&iter)) {
+      uint32_t len;
+      const uint8_t *data;
+      bson_iter_document (&iter, &len, &data);
+      BSON_ASSERT (bson_init_static (out, data, len));
+
+      return true;
+   }
+   return false;
+}
+
+static bool
+_truncate_handshake (bson_t **doc)
+{
+   if ((*doc)->len > HANDSHAKE_MAX_SIZE) {
+      bson_t env_doc;
+      if (_get_subdoc_static (*doc, "env", &env_doc)) {
+         bson_t *new_env = bson_new ();
+         bson_copy_to_including_noinit (&env_doc, new_env, "name", NULL);
+
+         bson_t *new_doc = bson_new ();
+         bson_copy_to_excluding_noinit (*doc, new_doc, "env", NULL);
+
+         bson_append_document (new_doc, "env", -1, new_env);
+         bson_destroy (new_env);
+         bson_destroy (*doc);
+         *doc = new_doc;
+      }
+   }
+
+   if ((*doc)->len > HANDSHAKE_MAX_SIZE) {
+      bson_t os_doc;
+      if (_get_subdoc_static (*doc, "os", &os_doc)) {
+         bson_t *new_os = bson_new ();
+         bson_copy_to_including_noinit (&os_doc, new_os, "type", NULL);
+
+         bson_t *new_doc = bson_new ();
+         bson_copy_to_excluding_noinit (*doc, new_doc, "os", NULL);
+
+         bson_append_document (new_doc, "os", -1, new_os);
+         bson_destroy (new_os);
+         bson_destroy (*doc);
+         *doc = new_doc;
+      }
+   }
+
+   if ((*doc)->len > HANDSHAKE_MAX_SIZE) {
+      bson_t *new_doc = bson_new ();
+      bson_copy_to_excluding_noinit (*doc, new_doc, "env", NULL);
+      bson_destroy (*doc);
+      *doc = new_doc;
+   }
+
+   const mongoc_handshake_t *md = _mongoc_handshake_get ();
+   if ((*doc)->len > HANDSHAKE_MAX_SIZE && md->platform) {
+      bson_t *new_doc = bson_new ();
+      bson_copy_to_excluding_noinit (*doc, new_doc, "platform", NULL);
+      _append_platform_field (new_doc, md->platform, true);
+      bson_destroy (*doc);
+      *doc = new_doc;
+   }
+
+   return (*doc)->len <= HANDSHAKE_MAX_SIZE;
 }
 
 /*
  * Return true if we build the document, and it's not too big
  * false if there's no way to prevent the doc from being too big. In this
- * case, the caller shouldn't include it with isMaster
+ * case, the caller shouldn't include it with hello
  */
-bool
-_mongoc_handshake_build_doc_with_application (bson_t *doc, const char *appname)
+bson_t *
+_mongoc_handshake_build_doc_with_application (const char *appname)
 {
    const mongoc_handshake_t *md = _mongoc_handshake_get ();
-   bson_t child;
-
-   if (appname) {
-      BSON_APPEND_DOCUMENT_BEGIN (doc, "application", &child);
-      BSON_APPEND_UTF8 (&child, "name", appname);
-      bson_append_document_end (doc, &child);
+   char *env_name = NULL;
+   switch (md->env) {
+   case MONGOC_HANDSHAKE_ENV_AWS:
+      env_name = "aws.lambda";
+      break;
+   case MONGOC_HANDSHAKE_ENV_GCP:
+      env_name = "gcp.func";
+      break;
+   case MONGOC_HANDSHAKE_ENV_AZURE:
+      env_name = "azure.func";
+      break;
+   case MONGOC_HANDSHAKE_ENV_VERCEL:
+      env_name = "vercel";
+      break;
+   case MONGOC_HANDSHAKE_ENV_NONE:
+      env_name = NULL;
+      break;
+   default:
+      break;
    }
 
-   BSON_APPEND_DOCUMENT_BEGIN (doc, "driver", &child);
-   BSON_APPEND_UTF8 (&child, "name", md->driver_name);
-   BSON_APPEND_UTF8 (&child, "version", md->driver_version);
-   bson_append_document_end (doc, &child);
-
-   BSON_APPEND_DOCUMENT_BEGIN (doc, "os", &child);
-
-   BSON_ASSERT (md->os_type);
-   BSON_APPEND_UTF8 (&child, "type", md->os_type);
-
-   if (md->os_name) {
-      BSON_APPEND_UTF8 (&child, "name", md->os_name);
-   }
-
-   if (md->os_version) {
-      BSON_APPEND_UTF8 (&child, "version", md->os_version);
-   }
-
-   if (md->os_architecture) {
-      BSON_APPEND_UTF8 (&child, "architecture", md->os_architecture);
-   }
-
-   bson_append_document_end (doc, &child);
-
-   if (doc->len > HANDSHAKE_MAX_SIZE) {
-      /* We've done all we can possibly do to ensure the current
-       * document is below the maxsize, so if it overflows there is
-       * nothing else we can do, so we fail */
-      return false;
-   }
+   bson_t *doc = bson_new ();
+   // Optimistically include all handshake data
+   bsonBuildAppend (
+      *doc,
+      if (appname,
+          then (kv ("application", doc (kv ("name", cstr (appname)))))),
+      kv ("driver",
+          doc (kv ("name", cstr (md->driver_name)),
+               kv ("version", cstr (md->driver_version)))),
+      kv (
+         "os",
+         doc (kv ("type", cstr (md->os_type)),
+              if (md->os_name, then (kv ("name", cstr (md->os_name)))),
+              if (md->os_version, then (kv ("version", cstr (md->os_version)))),
+              if (md->os_architecture,
+                  then (kv ("architecture", cstr (md->os_architecture)))))),
+      if (env_name,
+          then (kv (
+             "env",
+             doc (kv ("name", cstr (env_name)),
+                  if (md->env_timeout_sec.set,
+                      then (kv ("timeout_sec",
+                                int32 (md->env_timeout_sec.value)))),
+                  if (md->env_memory_mb.set,
+                      then (kv ("memory_mb", int32 (md->env_memory_mb.value)))),
+                  if (md->env_region,
+                      then (kv ("region", cstr (md->env_region)))))))));
 
    if (md->platform) {
-      _append_platform_field (doc, md->platform);
+      _append_platform_field (doc, md->platform, false);
    }
 
-   return true;
+   if (_truncate_handshake (&doc)) {
+      return doc;
+   } else {
+      bson_destroy (doc);
+      return NULL;
+   }
 }
 
 void
@@ -572,32 +754,30 @@ _mongoc_handshake_freeze (void)
 /*
  * free (*s) and make *s point to *s concated with suffix.
  * If *s is NULL it's treated like it's an empty string.
- * If suffix is NULL, nothing happens.
  */
 static void
-_append_and_truncate (char **s, const char *suffix, int max_len)
+_append_and_truncate (char **s, const char *suffix, size_t max_len)
 {
    char *old_str = *s;
-   char *prefix;
-   const int delim_len = (int) strlen (" / ");
-   int space_for_suffix;
+   const size_t delim_len = strlen (" / ");
 
-   BSON_ASSERT (s);
+   BSON_ASSERT_PARAM (s);
+   BSON_ASSERT_PARAM (suffix);
 
-   prefix = old_str ? old_str : "";
+   const char *const prefix = old_str ? old_str : "";
 
-   if (!suffix) {
-      return;
-   }
+   const size_t required_space = strlen (prefix) + delim_len;
 
-   space_for_suffix = max_len - (int) strlen (prefix) - delim_len;
-
-   if (space_for_suffix <= 0) {
+   if (max_len <= required_space) {
       /* the old string already takes the whole allotted space */
       return;
    }
 
-   *s = bson_strdup_printf ("%s / %.*s", prefix, space_for_suffix, suffix);
+   const size_t space_for_suffix = max_len - required_space;
+   BSON_ASSERT (bson_in_range_unsigned (int, space_for_suffix));
+
+   *s =
+      bson_strdup_printf ("%s / %.*s", prefix, (int) space_for_suffix, suffix);
    BSON_ASSERT (strlen (*s) <= max_len);
 
    bson_free (old_str);
@@ -606,10 +786,10 @@ _append_and_truncate (char **s, const char *suffix, int max_len)
 
 /*
  * Set some values in our global handshake struct. These values will be sent
- * to the server as part of the initial connection handshake (isMaster).
+ * to the server as part of the initial connection handshake (hello).
  * If this function is called more than once, or after we've connected to a
- * mongod, then it will do nothing and return false. It will return true if it
- * successfully sets the values.
+ * mongod, then it will do nothing and return false. It will return true if
+ * it successfully sets the values.
  *
  * All arguments are optional.
  */
@@ -627,29 +807,38 @@ mongoc_handshake_data_append (const char *driver_name,
       return false;
    }
 
+   BSON_ASSERT (_mongoc_handshake_get ()->platform);
+
    /* allow practically any size for "platform", we'll trim it down in
     * _mongoc_handshake_build_doc_with_application */
    platform_space =
       HANDSHAKE_MAX_SIZE - (int) strlen (_mongoc_handshake_get ()->platform);
 
-   /* we check for an empty string as a special case to avoid an unnecessary
-    * delimiter being added in front of the string by _append_and_truncate */
-   if (strcmp (_mongoc_handshake_get ()->platform, "") == 0) {
-      bson_free (_mongoc_handshake_get ()->platform);
-      _mongoc_handshake_get ()->platform =
-         bson_strdup_printf ("%.*s", platform_space, platform);
-   } else {
-      _append_and_truncate (
-         &_mongoc_handshake_get ()->platform, platform, HANDSHAKE_MAX_SIZE);
+   if (platform) {
+      /* we check for an empty string as a special case to avoid an
+       * unnecessary delimiter being added in front of the string by
+       * _append_and_truncate */
+      if (_mongoc_handshake_get ()->platform[0] == '\0') {
+         bson_free (_mongoc_handshake_get ()->platform);
+         _mongoc_handshake_get ()->platform =
+            bson_strdup_printf ("%.*s", platform_space, platform);
+      } else {
+         _append_and_truncate (
+            &_mongoc_handshake_get ()->platform, platform, HANDSHAKE_MAX_SIZE);
+      }
    }
 
-   _append_and_truncate (&_mongoc_handshake_get ()->driver_name,
-                         driver_name,
-                         HANDSHAKE_DRIVER_NAME_MAX);
+   if (driver_name) {
+      _append_and_truncate (&_mongoc_handshake_get ()->driver_name,
+                            driver_name,
+                            HANDSHAKE_DRIVER_NAME_MAX);
+   }
 
-   _append_and_truncate (&_mongoc_handshake_get ()->driver_version,
-                         driver_version,
-                         HANDSHAKE_DRIVER_VERSION_MAX);
+   if (driver_version) {
+      _append_and_truncate (&_mongoc_handshake_get ()->driver_version,
+                            driver_version,
+                            HANDSHAKE_DRIVER_VERSION_MAX);
+   }
 
    _mongoc_handshake_freeze ();
    bson_mutex_unlock (&gHandshakeLock);
@@ -684,25 +873,15 @@ _mongoc_handshake_append_sasl_supported_mechs (const mongoc_uri_t *uri,
 
 void
 _mongoc_handshake_parse_sasl_supported_mechs (
-   const bson_t *ismaster,
+   const bson_t *hello,
    mongoc_handshake_sasl_supported_mechs_t *sasl_supported_mechs)
 {
-   bson_iter_t iter;
    memset (sasl_supported_mechs, 0, sizeof (*sasl_supported_mechs));
-   if (bson_iter_init_find (&iter, ismaster, "saslSupportedMechs")) {
-      bson_iter_t array_iter;
-      if (BSON_ITER_HOLDS_ARRAY (&iter) &&
-          bson_iter_recurse (&iter, &array_iter)) {
-         while (bson_iter_next (&array_iter)) {
-            if (BSON_ITER_HOLDS_UTF8 (&array_iter)) {
-               const char *mechanism_name = bson_iter_utf8 (&array_iter, NULL);
-               if (0 == strcmp (mechanism_name, "SCRAM-SHA-256")) {
-                  sasl_supported_mechs->scram_sha_256 = true;
-               } else if (0 == strcmp (mechanism_name, "SCRAM-SHA-1")) {
-                  sasl_supported_mechs->scram_sha_1 = true;
-               }
-            }
-         }
-      }
-   }
+   bsonParse (*hello,
+              find (keyWithType ("saslSupportedMechs", array),
+                    visitEach (case (
+                       when (strEqual ("SCRAM-SHA-256"),
+                             do (sasl_supported_mechs->scram_sha_256 = true)),
+                       when (strEqual ("SCRAM-SHA-1"),
+                             do (sasl_supported_mechs->scram_sha_1 = true))))));
 }
