@@ -14,12 +14,12 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-prelude.h"
+#include "mongoc-prelude.h"
 
 #ifndef MONGOC_HOST_LIST_PRIVATE_H
 #define MONGOC_HOST_LIST_PRIVATE_H
 
-#include "mongoc/mongoc-host-list.h"
+#include "mongoc-host-list.h"
 
 
 BSON_BEGIN_DECLS
@@ -30,9 +30,12 @@ _mongoc_host_list_push (const char *host,
                         int family,
                         mongoc_host_list_t *next);
 
+void
+_mongoc_host_list_upsert (mongoc_host_list_t **list,
+                          const mongoc_host_list_t *new_host);
+
 mongoc_host_list_t *
-_mongoc_host_list_copy (const mongoc_host_list_t *src,
-                        mongoc_host_list_t *next);
+_mongoc_host_list_copy_all (const mongoc_host_list_t *src);
 
 bool
 _mongoc_host_list_from_string (mongoc_host_list_t *host_list,
@@ -49,9 +52,12 @@ _mongoc_host_list_from_hostport_with_err (mongoc_host_list_t *host_list,
                                           uint16_t port,
                                           bson_error_t *error);
 
+size_t
+_mongoc_host_list_length (const mongoc_host_list_t *list);
+
 bool
-_mongoc_host_list_equal (const mongoc_host_list_t *host_a,
-                         const mongoc_host_list_t *host_b);
+_mongoc_host_list_compare_one (const mongoc_host_list_t *host_a,
+                               const mongoc_host_list_t *host_b);
 
 void
 _mongoc_host_list_remove_host (mongoc_host_list_t **phosts,
@@ -61,6 +67,10 @@ _mongoc_host_list_remove_host (mongoc_host_list_t **phosts,
 void
 _mongoc_host_list_destroy_all (mongoc_host_list_t *host);
 
+bool
+_mongoc_host_list_contains_one (mongoc_host_list_t *host_list,
+                                mongoc_host_list_t *host);
+
 BSON_END_DECLS
 
 
