@@ -170,7 +170,6 @@ _mongoc_topology_scanner_add_speculative_authentication (
    bson_t *cmd,
    const mongoc_uri_t *uri,
    const mongoc_ssl_opt_t *ssl_opts,
-   mongoc_scram_cache_t *scram_cache,
    mongoc_scram_t *scram /* OUT */)
 {
    bson_t auth_cmd;
@@ -204,10 +203,6 @@ _mongoc_topology_scanner_add_speculative_authentication (
 
       _mongoc_uri_init_scram (uri, scram, algo);
 
-      if (scram_cache) {
-         _mongoc_scram_set_cache (scram, scram_cache);
-      }
-
       if (_mongoc_cluster_get_auth_cmd_scram (algo, scram, &auth_cmd, &error)) {
          const char *auth_source;
 
@@ -258,41 +253,33 @@ _build_handshake_cmd (const bson_t *basis_cmd,
                       bool is_loadbalanced)
 {
    bson_t *doc = bson_copy (basis_cmd);
-   bson_t subdoc;
    bson_iter_t iter;
-   const char *key;
    const bson_t *compressors;
-   int count = 0;
-   char buf[16];
-   bool subdoc_okay;
+   bson_array_builder_t *subarray;
 
    BSON_ASSERT (doc);
+   bson_t *handshake_doc =
+      _mongoc_handshake_build_doc_with_application (appname);
 
-   BSON_APPEND_DOCUMENT_BEGIN (doc, HANDSHAKE_FIELD, &subdoc);
-   subdoc_okay =
-      _mongoc_handshake_build_doc_with_application (&subdoc, appname);
-   bson_append_document_end (doc, &subdoc);
-
-   if (!subdoc_okay) {
+   if (!handshake_doc) {
       bson_destroy (doc);
       return NULL;
    }
+   bson_append_document (doc, HANDSHAKE_FIELD, -1, handshake_doc);
+   bson_destroy (handshake_doc);
 
-   BSON_APPEND_ARRAY_BEGIN (doc, "compression", &subdoc);
+   BSON_APPEND_ARRAY_BUILDER_BEGIN (doc, "compression", &subarray);
    if (uri) {
       compressors = mongoc_uri_get_compressors (uri);
 
       if (bson_iter_init (&iter, compressors)) {
          while (bson_iter_next (&iter)) {
-            const size_t keylen =
-               bson_uint32_to_string (count++, &key, buf, sizeof (buf));
-            BSON_ASSERT (bson_in_range_unsigned (int, keylen));
-            bson_append_utf8 (
-               &subdoc, key, (int) keylen, bson_iter_key (&iter), -1);
+            bson_array_builder_append_utf8 (
+               subarray, bson_iter_key (&iter), -1);
          }
       }
    }
-   bson_append_array_end (doc, &subdoc);
+   bson_append_array_builder_end (doc, subarray);
 
    if (is_loadbalanced) {
       BSON_APPEND_BOOL (doc, "loadBalanced", true);
@@ -419,12 +406,8 @@ _begin_hello_cmd (mongoc_topology_scanner_node_t *node,
       ssl_opts = ts->ssl_opts;
 #endif
 
-      // _mongoc_topology_scanner_add_speculative_authentication is called with
-      // NULL for the scram_cache argument. The scram cache is not used for
-      // speculative authentication in the topology scanner. This is planned to
-      // be improved in CDRIVER-3642.
       _mongoc_topology_scanner_add_speculative_authentication (
-         &cmd, ts->uri, ssl_opts, NULL, &node->scram);
+         &cmd, ts->uri, ssl_opts, &node->scram);
    }
 
    if (!bson_empty (&ts->cluster_time)) {
