@@ -48,7 +48,7 @@ typedef bool (*hello_callback_func_t) (request_t *request,
 typedef void (*destructor_t) (void *data);
 
 mock_server_t *
-mock_server_new ();
+mock_server_new (void);
 
 mock_server_t *
 mock_server_with_auto_hello (int32_t max_wire_version);
@@ -103,9 +103,6 @@ mock_server_get_host_and_port (mock_server_t *server);
 uint16_t
 mock_server_get_port (mock_server_t *server);
 
-int64_t
-mock_server_get_request_timeout_msec (mock_server_t *server);
-
 void
 mock_server_set_request_timeout_msec (mock_server_t *server,
                                       int64_t request_timeout_msec);
@@ -116,9 +113,6 @@ mock_server_get_rand_delay (mock_server_t *server);
 void
 mock_server_set_rand_delay (mock_server_t *server, bool rand_delay);
 
-double
-mock_server_get_uptime_sec (mock_server_t *server);
-
 request_t *
 mock_server_receives_request (mock_server_t *server);
 
@@ -129,11 +123,6 @@ mock_server_receives_command (mock_server_t *server,
                               const char *command_json,
                               ...);
 
-request_t *
-mock_server_matches_any_hello_with_json (request_t *request,
-                                         const char *match_json_op_msg,
-                                         const char *match_json_op_query);
-
 request_t *
 mock_server_receives_any_hello (mock_server_t *server);
 
@@ -161,37 +150,6 @@ mock_server_receives_query (mock_server_t *server,
                             const char *query_json,
                             const char *fields_json);
 
-request_t *
-mock_server_receives_insert (mock_server_t *server,
-                             const char *ns,
-                             mongoc_insert_flags_t flags,
-                             const char *doc_json);
-
-request_t *
-mock_server_receives_bulk_insert (mock_server_t *server,
-                                  const char *ns,
-                                  mongoc_insert_flags_t flags,
-                                  int n);
-
-request_t *
-mock_server_receives_update (mock_server_t *server,
-                             const char *ns,
-                             mongoc_update_flags_t flags,
-                             const char *selector_json,
-                             const char *update_json);
-
-request_t *
-mock_server_receives_delete (mock_server_t *server,
-                             const char *ns,
-                             mongoc_remove_flags_t flags,
-                             const char *selector_json);
-
-request_t *
-mock_server_receives_getmore (mock_server_t *server,
-                              const char *ns,
-                              int32_t n_return,
-                              int64_t cursor_id);
-
 request_t *
 mock_server_receives_kill_cursors (mock_server_t *server, int64_t cursor_id);
 
@@ -208,45 +166,45 @@ mock_server_receives_bulk_msg (mock_server_t *server,
                                size_t n_docs);
 
 void
-mock_server_hangs_up (request_t *request);
+reply_to_request_with_hang_up (request_t *request);
 
 void
-mock_server_resets (request_t *request);
+reply_to_request_with_reset (request_t *request);
 
 void
-mock_server_replies (request_t *request,
-                     mongoc_reply_flags_t flags,
-                     int64_t cursor_id,
-                     int32_t starting_from,
-                     int32_t number_returned,
-                     const char *docs_json);
+reply_to_request (request_t *request,
+                  mongoc_reply_flags_t flags,
+                  int64_t cursor_id,
+                  int32_t starting_from,
+                  int32_t number_returned,
+                  const char *docs_json);
 
 void
-mock_server_replies_simple (request_t *request, const char *docs_json);
+reply_to_request_simple (request_t *request, const char *docs_json);
 
 void
-mock_server_replies_ok_and_destroys (request_t *request);
+reply_to_request_with_ok_and_destroy (request_t *request);
 
 void
-mock_server_replies_to_find (request_t *request,
-                             mongoc_query_flags_t flags,
-                             int64_t cursor_id,
-                             int32_t number_returned,
-                             const char *ns,
-                             const char *reply_json,
-                             bool is_command);
+reply_to_find_request (request_t *request,
+                       mongoc_query_flags_t flags,
+                       int64_t cursor_id,
+                       int32_t number_returned,
+                       const char *ns,
+                       const char *reply_json,
+                       bool is_command);
 
 void
-mock_server_replies_opmsg (request_t *request,
-                           mongoc_op_msg_flags_t flags,
-                           const bson_t *doc);
+reply_to_op_msg_request (request_t *request,
+                         mongoc_op_msg_flags_t flags,
+                         const bson_t *doc);
 
 void
-mock_server_reply_multi (request_t *request,
-                         mongoc_reply_flags_t flags,
-                         const bson_t *docs,
-                         int n_docs,
-                         int64_t cursor_id);
+reply_to_request_with_multiple_docs (request_t *request,
+                                     mongoc_reply_flags_t flags,
+                                     const bson_t *docs,
+                                     int n_docs,
+                                     int64_t cursor_id);
 
 void
 mock_server_destroy (mock_server_t *server);
