@@ -26,7 +26,7 @@ server_id_for_reads (mongoc_cluster_t *cluster)
    uint32_t id;
 
    server_stream =
-      mongoc_cluster_stream_for_reads (cluster, NULL, NULL, NULL, &error);
+      mongoc_cluster_stream_for_reads (cluster, NULL, NULL, NULL, NULL, &error);
    ASSERT_OR_PRINT (server_stream, error);
    id = server_stream->sd->id;
 
@@ -1140,8 +1140,8 @@ future_command_private (mongoc_client_t *client)
 
    ASSERT (client);
 
-   server_stream =
-      mongoc_cluster_stream_for_writes (&client->cluster, NULL, NULL, &error);
+   server_stream = mongoc_cluster_stream_for_writes (
+      &client->cluster, NULL, NULL, NULL, &error);
    ASSERT_OR_PRINT (server_stream, error);
 
    mongoc_cmd_parts_init (
@@ -1785,8 +1785,11 @@ test_cluster_stream_invalidation_single (void)
 
    /* Test "clearing the pool". This should invalidate existing server streams.
     */
-   stream = mongoc_cluster_stream_for_writes (
-      &client->cluster, NULL /* session */, NULL /* reply */, &error);
+   stream = mongoc_cluster_stream_for_writes (&client->cluster,
+                                              NULL /* session */,
+                                              NULL /* deprioritized servers */,
+                                              NULL /* reply */,
+                                              &error);
    ASSERT_OR_PRINT (stream, error);
    BSON_ASSERT (mongoc_cluster_stream_valid (&client->cluster, stream));
    tdmod = mc_tpld_modify_begin (client->topology);
@@ -1798,8 +1801,11 @@ test_cluster_stream_invalidation_single (void)
 
    /* Test closing the connection. This should invalidate existing server
     * streams. */
-   stream = mongoc_cluster_stream_for_writes (
-      &client->cluster, NULL /* session */, NULL /* reply */, &error);
+   stream = mongoc_cluster_stream_for_writes (&client->cluster,
+                                              NULL /* session */,
+                                              NULL /* deprioritized servers */,
+                                              NULL /* reply */,
+                                              &error);
    ASSERT_OR_PRINT (stream, error);
    BSON_ASSERT (mongoc_cluster_stream_valid (&client->cluster, stream));
    mongoc_cluster_disconnect_node (&client->cluster, sd->id);
@@ -1807,8 +1813,11 @@ test_cluster_stream_invalidation_single (void)
    mongoc_server_stream_cleanup (stream);
 
    /* Test that a new stream is considered valid. */
-   stream = mongoc_cluster_stream_for_writes (
-      &client->cluster, NULL /* session */, NULL /* reply */, &error);
+   stream = mongoc_cluster_stream_for_writes (&client->cluster,
+                                              NULL /* session */,
+                                              NULL /* deprioritized servers */,
+                                              NULL /* reply */,
+                                              &error);
    ASSERT_OR_PRINT (stream, error);
    BSON_ASSERT (mongoc_cluster_stream_valid (&client->cluster, stream));
    mongoc_server_stream_cleanup (stream);
@@ -1837,8 +1846,11 @@ test_cluster_stream_invalidation_pooled (void)
 
    /* Test "clearing the pool". This should invalidate existing server streams.
     */
-   stream = mongoc_cluster_stream_for_writes (
-      &client->cluster, NULL /* session */, NULL /* reply */, &error);
+   stream = mongoc_cluster_stream_for_writes (&client->cluster,
+                                              NULL /* session */,
+                                              NULL /* deprioritized servers */,
+                                              NULL /* reply */,
+                                              &error);
    ASSERT_OR_PRINT (stream, error);
    BSON_ASSERT (mongoc_cluster_stream_valid (&client->cluster, stream));
    tdmod = mc_tpld_modify_begin (client->topology);
@@ -1850,8 +1862,11 @@ test_cluster_stream_invalidation_pooled (void)
 
    /* Test closing the connection. This should invalidate existing server
     * streams. */
-   stream = mongoc_cluster_stream_for_writes (
-      &client->cluster, NULL /* session */, NULL /* reply */, &error);
+   stream = mongoc_cluster_stream_for_writes (&client->cluster,
+                                              NULL /* session */,
+                                              NULL /* deprioritized servers */,
+                                              NULL /* reply */,
+                                              &error);
    ASSERT_OR_PRINT (stream, error);
    BSON_ASSERT (mongoc_cluster_stream_valid (&client->cluster, stream));
    mongoc_cluster_disconnect_node (&client->cluster, sd->id);
@@ -1859,8 +1874,11 @@ test_cluster_stream_invalidation_pooled (void)
    mongoc_server_stream_cleanup (stream);
 
    /* Test that a new stream is considered valid. */
-   stream = mongoc_cluster_stream_for_writes (
-      &client->cluster, NULL /* session */, NULL /* reply */, &error);
+   stream = mongoc_cluster_stream_for_writes (&client->cluster,
+                                              NULL /* session */,
+                                              NULL /* deprioritized servers */,
+                                              NULL /* reply */,
+                                              &error);
    ASSERT_OR_PRINT (stream, error);
    BSON_ASSERT (mongoc_cluster_stream_valid (&client->cluster, stream));
    mongoc_server_stream_cleanup (stream);
