@@ -46,29 +46,25 @@ _assert_options_match (const bson_t *test, mongoc_uri_t *uri)
                                                                 : opts_from_uri;
       if (!bson_iter_init_find_case (
              &uri_opts_iter, opts_or_creds, opt_name_canon)) {
-         fprintf (stderr,
-                  "URI options incorrectly set from TXT record: "
-                  "no option named \"%s\"\n"
-                  "expected: %s\n"
-                  "actual: %s\n",
-                  opt_name,
-                  bson_as_json (&opts_from_test, NULL),
-                  bson_as_json (opts_or_creds, NULL));
-         abort ();
+         test_error ("URI options incorrectly set from TXT record: "
+                     "no option named \"%s\"\n"
+                     "expected: %s\n"
+                     "actual: %s",
+                     opt_name,
+                     bson_as_json (&opts_from_test, NULL),
+                     bson_as_json (opts_or_creds, NULL));
       }
 
       test_value = bson_iter_value (&test_opts_iter);
       uri_value = bson_iter_value (&uri_opts_iter);
       if (!match_bson_value (uri_value, test_value, &ctx)) {
-         fprintf (stderr,
-                  "URI option \"%s\" incorrectly set from TXT record: %s\n"
-                  "expected: %s\n"
-                  "actual: %s\n",
-                  opt_name,
-                  ctx.errmsg,
-                  bson_as_json (&opts_from_test, NULL),
-                  bson_as_json (opts_from_uri, NULL));
-         abort ();
+         test_error ("URI option \"%s\" incorrectly set from TXT record: %s\n"
+                     "expected: %s\n"
+                     "actual: %s",
+                     opt_name,
+                     ctx.errmsg,
+                     bson_as_json (&opts_from_test, NULL),
+                     bson_as_json (opts_from_uri, NULL));
       }
    }
 }
@@ -122,12 +118,12 @@ host_list_contains (const mongoc_host_list_t *hl, const char *host_and_port)
 }
 
 
-static int
+static int64_t
 hosts_count (const bson_t *test)
 {
    bson_iter_t iter;
    bson_iter_t hosts;
-   int c = 0;
+   int64_t c = 0;
 
    if (bson_iter_init_find (&iter, test, "hosts")) {
       BSON_ASSERT (bson_iter_recurse (&iter, &hosts));
@@ -171,16 +167,15 @@ _host_list_matches (const bson_t *test, context_t *ctx)
    }
 
    else if (bson_iter_init_find (&iter, test, "numHosts")) {
-      const int expected = bson_iter_as_int64 (&iter);
-      int actual = 0;
+      const int64_t expected = bson_iter_as_int64 (&iter);
 
       bson_mutex_lock (&ctx->mutex);
-      actual = _mongoc_host_list_length (ctx->hosts);
+      const size_t actual = _mongoc_host_list_length (ctx->hosts);
       _mongoc_host_list_destroy_all (ctx->hosts);
       ctx->hosts = NULL;
       bson_mutex_unlock (&ctx->mutex);
 
-      ret = expected == actual;
+      ret = bson_cmp_equal_su (expected, actual);
    }
 
    return ret;
@@ -225,16 +220,14 @@ _test_dns_maybe_pooled (bson_t *test, bool pooled)
 #ifdef MONGOC_ENABLE_SSL
    mongoc_ssl_opt_t ssl_opts;
 #endif
-   int n_hosts;
    bson_error_t error;
    bool r;
    const char *uri_str;
 
    if (!test_framework_get_ssl ()) {
-      fprintf (stderr,
-               "Must configure an SSL replica set and set MONGOC_TEST_SSL=on "
-               "and other ssl options to test DNS\n");
-      abort ();
+      test_error (
+         "Must configure an SSL replica set and set MONGOC_TEST_SSL=on "
+         "and other ssl options to test DNS");
    }
 
    uri_str = bson_lookup_utf8 (test, "uri");
@@ -318,10 +311,10 @@ _test_dns_maybe_pooled (bson_t *test, bool pooled)
    BSON_ASSERT (client->ssl_opts.allow_invalid_hostname);
 #endif
 
-   n_hosts = hosts_count (test);
+   const int64_t n_hosts = hosts_count (test);
 
    if (pooled) {
-      if (n_hosts && !expect_error) {
+      if (n_hosts > 0 && !expect_error) {
          WAIT_UNTIL (_host_list_matches (test, &ctx));
       } else {
          r = mongoc_client_command_simple (
@@ -337,7 +330,7 @@ _test_dns_maybe_pooled (bson_t *test, bool pooled)
        * connections need to authenticate, and the credentials in the tests do
        * not correspond to the test users. TODO (CDRIVER-4046): unskip these
        * tests. */
-      if (n_hosts && !expect_error) {
+      if (n_hosts > 0 && !expect_error) {
          r = mongoc_client_command_simple (
             client, "admin", tmp_bson ("{'ping': 1}"), NULL, NULL, &error);
          ASSERT_OR_PRINT (r, error);
@@ -493,13 +486,12 @@ dump_hosts (mongoc_host_list_t *hosts)
 static void
 dump_topology_description (const mongoc_topology_description_t *td)
 {
-   size_t i;
    const mongoc_server_description_t *sd;
    const mongoc_set_t *servers = mc_tpld_servers_const (td);
 
    MONGOC_DEBUG ("topology hosts:");
-   for (i = 0; i < servers->items_len; ++i) {
-      sd = mongoc_set_get_item_const (servers, (int) i);
+   for (size_t i = 0u; i < servers->items_len; ++i) {
+      sd = mongoc_set_get_item_const (servers, i);
       MONGOC_DEBUG ("- %s", sd->host.host_and_port);
    }
 }
@@ -508,19 +500,19 @@ static void
 check_topology_description (mongoc_topology_description_t *td,
                             mongoc_host_list_t *hosts)
 {
-   int nhosts = 0;
+   size_t nhosts = 0u;
    mongoc_host_list_t *host;
    const mongoc_set_t *servers = mc_tpld_servers_const (td);
 
    for (host = hosts; host; host = host->next) {
-      uint32_t server_count;
+      ++nhosts;
 
-      nhosts++;
       /* Check that "host" is already in the topology description by upserting
        * it, and ensuring that the number of servers remains constant. */
-      server_count = servers->items_len;
+      const size_t server_count = servers->items_len;
       BSON_ASSERT (mongoc_topology_description_add_server (
          td, host->host_and_port, NULL));
+
       if (server_count != servers->items_len) {
          dump_topology_description (td);
          dump_hosts (hosts);
@@ -674,6 +666,8 @@ _prose_test_ping (mongoc_client_t *client)
    bson_error_t error;
    bson_t *cmd = BCON_NEW ("ping", BCON_INT32 (1));
 
+   ASSERT (client);
+
    if (!mongoc_client_command_simple (
           client, "admin", cmd, NULL, NULL, &error)) {
       test_error ("ping failed: %s", error.message);
@@ -916,10 +910,13 @@ _mock_rr_resolver_prose_test_10 (const char *service,
    BSON_ASSERT_PARAM (error);
 
    if (rr_type == MONGOC_RR_SRV) {
+      const size_t count = _mongoc_host_list_length (rr_data->hosts);
+      BSON_ASSERT (bson_in_range_unsigned (uint32_t, count));
+
       rr_data->hosts = MAKE_HOSTS ("localhost.test.build.10gen.cc:27017",
                                    "localhost.test.build.10gen.cc:27019",
                                    "localhost.test.build.10gen.cc:27020");
-      rr_data->count = _mongoc_host_list_length (rr_data->hosts);
+      rr_data->count = (uint32_t) count;
       rr_data->min_ttl = 0u;
       rr_data->txt_record_opts = NULL;
    }
@@ -1017,9 +1014,12 @@ _mock_rr_resolver_prose_test_11 (const char *service,
    BSON_ASSERT_PARAM (error);
 
    if (rr_type == MONGOC_RR_SRV) {
+      const size_t count = _mongoc_host_list_length (rr_data->hosts);
+      BSON_ASSERT (bson_in_range_unsigned (uint32_t, count));
+
       rr_data->hosts = MAKE_HOSTS ("localhost.test.build.10gen.cc:27019",
                                    "localhost.test.build.10gen.cc:27020");
-      rr_data->count = _mongoc_host_list_length (rr_data->hosts);
+      rr_data->count = (uint32_t) count;
       rr_data->min_ttl = 0u;
       rr_data->txt_record_opts = NULL;
    }
@@ -1116,10 +1116,13 @@ _mock_rr_resolver_prose_test_12 (const char *service,
    BSON_ASSERT_PARAM (error);
 
    if (rr_type == MONGOC_RR_SRV) {
+      const size_t count = _mongoc_host_list_length (rr_data->hosts);
+      BSON_ASSERT (bson_in_range_unsigned (uint32_t, count));
+
       rr_data->hosts = MAKE_HOSTS ("localhost.test.build.10gen.cc:27017",
                                    "localhost.test.build.10gen.cc:27019",
                                    "localhost.test.build.10gen.cc:27020");
-      rr_data->count = _mongoc_host_list_length (rr_data->hosts);
+      rr_data->count = (uint32_t) count;
       rr_data->min_ttl = 0u;
       rr_data->txt_record_opts = NULL;
    }
