@@ -87,8 +87,7 @@ _test_command_error (int32_t error_api_version)
       client, "db", tmp_bson ("{'foo': 1}"), NULL, &reply, &error);
    request = mock_server_receives_msg (
       server, MONGOC_MSG_NONE, tmp_bson ("{'$db': 'db', 'foo': 1}"));
-   mock_server_replies_simple (request,
-                               "{'ok': 0, 'code': 42, 'errmsg': 'foo'}");
+   reply_to_request_simple (request, "{'ok': 0, 'code': 42, 'errmsg': 'foo'}");
    ASSERT (!future_get_bool (future));
 
    if (error_api_version >= 2) {
