@@ -113,6 +113,8 @@ set_client_callbacks (mongoc_client_t *client)
    mongoc_apm_callbacks_t *cbs;
    stats_t *stats;
 
+   ASSERT (client);
+
    stats = bson_malloc0 (sizeof (stats_t));
    cbs = make_callbacks ();
    mongoc_client_set_apm_callbacks (client, cbs, stats);
@@ -126,6 +128,8 @@ set_client_pool_callbacks (mongoc_client_pool_t *pool)
    mongoc_apm_callbacks_t *cbs;
    stats_t *stats;
 
+   ASSERT (pool);
+
    stats = bson_malloc0 (sizeof (stats_t));
    cbs = make_callbacks ();
    mongoc_client_pool_set_apm_callbacks (pool, cbs, stats);
@@ -372,7 +376,7 @@ test_loadbalanced_cooldown_is_bypassed_single (void *unused)
       "admin",
       tmp_bson ("{'configureFailPoint': 'failCommand', 'mode': { 'times': 2 }, "
                 "'data': {'closeConnection': true, 'failCommands': ['ping', "
-                "'isMaster']}}"),
+                "'hello']}}"),
       NULL /* read prefs */,
       NULL /* reply */,
       &error);
@@ -389,7 +393,7 @@ test_loadbalanced_cooldown_is_bypassed_single (void *unused)
       error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "socket error");
 
    /* The next attempted command should attempt to scan, and fail when
-    * performing the handshake with the isMaster command. */
+    * performing the handshake with the hello command. */
    ok = mongoc_client_command_simple (client,
                                       "admin",
                                       tmp_bson ("{'ping': 1}"),
@@ -467,11 +471,11 @@ test_loadbalanced_handshake_sends_loadbalanced (void)
                                           &error);
    request = mock_server_receives_any_hello_with_match (
       server, "{'loadBalanced': true}", "{'loadBalanced': true}");
-   mock_server_replies_simple (request, LB_HELLO);
+   reply_to_request_simple (request, LB_HELLO);
    request_destroy (request);
 
    request = mock_server_receives_msg (server, 0, tmp_bson ("{'ping': 1}"));
-   mock_server_replies_ok_and_destroys (request);
+   reply_to_request_with_ok_and_destroy (request);
 
    ASSERT_OR_PRINT (future_get_bool (future), error);
    future_destroy (future);
@@ -529,7 +533,7 @@ test_loadbalanced_handshake_rejects_non_loadbalanced (void)
                                           &error);
    request = mock_server_receives_any_hello_with_match (
       server, "{'loadBalanced': true}", "{'loadBalanced': true}");
-   mock_server_replies_simple (request, NON_LB_HELLO);
+   reply_to_request_simple (request, NON_LB_HELLO);
    request_destroy (request);
    BSON_ASSERT (!future_get_bool (future));
    future_destroy (future);
@@ -558,6 +562,7 @@ test_pre_handshake_error_does_not_clear_pool (void)
    future_t *future;
    request_t *request;
    bson_error_t error;
+   const bson_t *match_loadBalanced = tmp_bson ("{'loadBalanced': true}");
 
    server = mock_server_new ();
    mock_server_auto_endsessions (server);
@@ -576,15 +581,17 @@ test_pre_handshake_error_does_not_clear_pool (void)
                                           NULL /* reply */,
                                           &error);
    /* A new connection is opened. */
-   request =
-      mock_server_receives_legacy_hello (server, "{'loadBalanced': true}");
+   request = mock_server_receives_hello_op_msg (server);
+   ASSERT (
+      request_matches_msg (request, MONGOC_MSG_NONE, &match_loadBalanced, 1));
+
    BSON_ASSERT (request);
-   mock_server_replies_simple (request, LB_HELLO);
+   reply_to_request_simple (request, LB_HELLO);
    request_destroy (request);
    /* The "ping" command is sent. */
    request = mock_server_receives_msg (server, 0, tmp_bson ("{'ping': 1}"));
    BSON_ASSERT (request);
-   mock_server_replies_ok_and_destroys (request);
+   reply_to_request_with_ok_and_destroy (request);
    ASSERT_OR_PRINT (future_get_bool (future), error);
    future_destroy (future);
 
@@ -597,11 +604,12 @@ test_pre_handshake_error_does_not_clear_pool (void)
                                           NULL /* reply */,
                                           &error);
    /* A new connection is opened. */
-   request =
-      mock_server_receives_legacy_hello (server, "{'loadBalanced': true}");
+   request = mock_server_receives_hello_op_msg (server);
+   ASSERT (
+      request_matches_msg (request, MONGOC_MSG_NONE, &match_loadBalanced, 1));
    BSON_ASSERT (request);
    capture_logs (true); /* hide Failed to buffer logs. */
-   mock_server_hangs_up (request);
+   reply_to_request_with_hang_up (request);
    request_destroy (request);
    BSON_ASSERT (!future_get_bool (future));
    ASSERT_ERROR_CONTAINS (
@@ -620,7 +628,7 @@ test_pre_handshake_error_does_not_clear_pool (void)
     * connection. The next command is the "ping". */
    request = mock_server_receives_msg (server, 0, tmp_bson ("{'ping': 1}"));
    BSON_ASSERT (request);
-   mock_server_replies_ok_and_destroys (request);
+   reply_to_request_with_ok_and_destroy (request);
    ASSERT_OR_PRINT (future_get_bool (future), error);
    future_destroy (future);
 
@@ -665,6 +673,7 @@ test_post_handshake_error_clears_pool (void)
    request_t *request;
    bson_error_t error;
    mongoc_server_description_t *monitor_sd;
+   const bson_t *match_loadBalanced = tmp_bson ("{'loadBalanced': true}");
 
    server = mock_server_new ();
    mock_server_auto_endsessions (server);
@@ -684,15 +693,16 @@ test_post_handshake_error_clears_pool (void)
                                           NULL /* reply */,
                                           &error);
    /* A new connection is opened. */
-   request =
-      mock_server_receives_legacy_hello (server, "{'loadBalanced': true}");
+   request = mock_server_receives_hello_op_msg (server);
+   ASSERT (
+      request_matches_msg (request, MONGOC_MSG_NONE, &match_loadBalanced, 1));
    BSON_ASSERT (request);
-   mock_server_replies_simple (request, LB_HELLO_A);
+   reply_to_request_simple (request, LB_HELLO_A);
    request_destroy (request);
    /* The "ping" command is sent. */
    request = mock_server_receives_msg (server, 0, tmp_bson ("{'ping': 1}"));
    BSON_ASSERT (request);
-   mock_server_replies_ok_and_destroys (request);
+   reply_to_request_with_ok_and_destroy (request);
    ASSERT_OR_PRINT (future_get_bool (future), error);
    future_destroy (future);
 
@@ -705,15 +715,16 @@ test_post_handshake_error_clears_pool (void)
                                           NULL /* reply */,
                                           &error);
    /* A new connection is opened. */
-   request =
-      mock_server_receives_legacy_hello (server, "{'loadBalanced': true}");
+   request = mock_server_receives_hello_op_msg (server);
+   ASSERT (
+      request_matches_msg (request, MONGOC_MSG_NONE, &match_loadBalanced, 1));
    BSON_ASSERT (request);
-   mock_server_replies_simple (request, LB_HELLO_A);
+   reply_to_request_simple (request, LB_HELLO_A);
    request_destroy (request);
    /* The "ping" command is sent. */
    request = mock_server_receives_msg (server, 0, tmp_bson ("{'ping': 1}"));
    BSON_ASSERT (request);
-   mock_server_replies_ok_and_destroys (request);
+   reply_to_request_with_ok_and_destroy (request);
    ASSERT_OR_PRINT (future_get_bool (future), error);
    future_destroy (future);
 
@@ -726,15 +737,16 @@ test_post_handshake_error_clears_pool (void)
                                           NULL /* reply */,
                                           &error);
    /* A new connection is opened. */
-   request =
-      mock_server_receives_legacy_hello (server, "{'loadBalanced': true}");
+   request = mock_server_receives_hello_op_msg (server);
+   ASSERT (
+      request_matches_msg (request, MONGOC_MSG_NONE, &match_loadBalanced, 1));
    BSON_ASSERT (request);
-   mock_server_replies_simple (request, LB_HELLO_B);
+   reply_to_request_simple (request, LB_HELLO_B);
    request_destroy (request);
    /* The "ping" command is sent. */
    request = mock_server_receives_msg (server, 0, tmp_bson ("{'ping': 1}"));
    BSON_ASSERT (request);
-   mock_server_replies_ok_and_destroys (request);
+   reply_to_request_with_ok_and_destroy (request);
    ASSERT_OR_PRINT (future_get_bool (future), error);
    future_destroy (future);
 
@@ -749,7 +761,7 @@ test_post_handshake_error_clears_pool (void)
    request = mock_server_receives_msg (server, 0, tmp_bson ("{'ping': 1}"));
    BSON_ASSERT (request);
    capture_logs (true); /* hide Failed to buffer logs. */
-   mock_server_hangs_up (request);
+   reply_to_request_with_hang_up (request);
    request_destroy (request);
    BSON_ASSERT (!future_get_bool (future));
    ASSERT_ERROR_CONTAINS (
@@ -769,15 +781,16 @@ test_post_handshake_error_clears_pool (void)
                                           NULL /* reply */,
                                           &error);
    /* A new connection is opened. */
-   request =
-      mock_server_receives_legacy_hello (server, "{'loadBalanced': true}");
+   request = mock_server_receives_hello_op_msg (server);
+   ASSERT (
+      request_matches_msg (request, MONGOC_MSG_NONE, &match_loadBalanced, 1));
    BSON_ASSERT (request);
-   mock_server_replies_simple (request, LB_HELLO_A);
+   reply_to_request_simple (request, LB_HELLO_A);
    request_destroy (request);
    /* The "ping" command is sent. */
    request = mock_server_receives_msg (server, 0, tmp_bson ("{'ping': 1}"));
    BSON_ASSERT (request);
-   mock_server_replies_ok_and_destroys (request);
+   reply_to_request_with_ok_and_destroy (request);
    ASSERT_OR_PRINT (future_get_bool (future), error);
    future_destroy (future);
 
@@ -791,7 +804,7 @@ test_post_handshake_error_clears_pool (void)
    /* The "ping" command is sent. */
    request = mock_server_receives_msg (server, 0, tmp_bson ("{'ping': 1}"));
    BSON_ASSERT (request);
-   mock_server_replies_ok_and_destroys (request);
+   reply_to_request_with_ok_and_destroy (request);
    ASSERT_OR_PRINT (future_get_bool (future), error);
    future_destroy (future);
 
@@ -810,6 +823,98 @@ skip_if_not_loadbalanced (void)
    return test_framework_is_loadbalanced () ? 1 : 0;
 }
 
+static void
+store_last_command_started_callback (const mongoc_apm_command_started_t *event)
+{
+   bson_t **last_command = mongoc_apm_command_started_get_context (event);
+   const bson_t *cmd = mongoc_apm_command_started_get_command (event);
+   bson_destroy (*last_command);
+   *last_command = bson_copy (cmd);
+}
+
+// `test_loadbalanced_sends_recoveryToken` is a regression test for
+// CDRIVER-4718. Ensure that a `recoveryToken` is included in the outgoing
+// `commitTransaction` and `abortTransaction` commands when connected to a load
+// balanced cluster.
+static void
+test_loadbalanced_sends_recoveryToken (void *unused)
+{
+   mongoc_client_t *client;
+   bson_error_t error;
+   bson_t *last_command = NULL;
+
+   BSON_UNUSED (unused);
+
+   client = test_framework_new_default_client ();
+   // Set a callback to store the most recent command started.
+   {
+      mongoc_apm_callbacks_t *cbs = mongoc_apm_callbacks_new ();
+      mongoc_apm_set_command_started_cb (cbs,
+                                         store_last_command_started_callback);
+      mongoc_client_set_apm_callbacks (client, cbs, &last_command);
+      mongoc_apm_callbacks_destroy (cbs);
+   }
+
+   mongoc_client_session_t *session =
+      mongoc_client_start_session (client, NULL /* opts */, &error);
+   ASSERT_OR_PRINT (session, error);
+
+   mongoc_collection_t *coll =
+      mongoc_client_get_collection (client, "db", "coll");
+
+   // Commit a transaction. Expect `commitTransaction` to include
+   // `recoveryToken`.
+   {
+      bool ok = mongoc_client_session_start_transaction (
+         session, NULL /* opts */, &error);
+      ASSERT_OR_PRINT (ok, error);
+
+      bson_t *insert_opts = tmp_bson ("{}");
+      ok = mongoc_client_session_append (session, insert_opts, &error);
+      ASSERT_OR_PRINT (ok, error);
+
+      ok = mongoc_collection_insert_one (
+         coll, tmp_bson ("{}"), insert_opts, NULL, &error);
+      ASSERT_OR_PRINT (ok, error);
+
+      ok = mongoc_client_session_commit_transaction (
+         session, NULL /* reply */, &error);
+      ASSERT_OR_PRINT (ok, error);
+
+      ASSERT_MATCH (
+         last_command,
+         "{'commitTransaction': 1, 'recoveryToken': { '$exists': true } }");
+   }
+
+   // Abort a transaction. Expect `abortTransaction` to include
+   // `recoveryToken`.
+   {
+      bool ok = mongoc_client_session_start_transaction (
+         session, NULL /* opts */, &error);
+      ASSERT_OR_PRINT (ok, error);
+
+      bson_t *insert_opts = tmp_bson ("{}");
+      ok = mongoc_client_session_append (session, insert_opts, &error);
+      ASSERT_OR_PRINT (ok, error);
+
+      ok = mongoc_collection_insert_one (
+         coll, tmp_bson ("{}"), insert_opts, NULL, &error);
+      ASSERT_OR_PRINT (ok, error);
+
+      ok = mongoc_client_session_abort_transaction (session, &error);
+      ASSERT_OR_PRINT (ok, error);
+
+      ASSERT_MATCH (
+         last_command,
+         "{'abortTransaction': 1, 'recoveryToken': { '$exists': true } }");
+   }
+
+   mongoc_collection_destroy (coll);
+   mongoc_client_session_destroy (session);
+   mongoc_client_destroy (client);
+   bson_destroy (last_command);
+}
+
 void
 test_loadbalanced_install (TestSuite *suite)
 {
@@ -880,4 +985,11 @@ test_loadbalanced_install (TestSuite *suite)
       suite,
       "/loadbalanced/post_handshake_error_clears_pool",
       test_post_handshake_error_clears_pool);
+
+   TestSuite_AddFull (suite,
+                      "/loadbalanced/sends_recoveryToken",
+                      test_loadbalanced_sends_recoveryToken,
+                      NULL /* ctx */,
+                      NULL /* dtor */,
+                      skip_if_not_loadbalanced);
 }
