@@ -160,11 +160,13 @@ test_getmore_iteration (mongoc_client_t *client)
    /* Store the primary ID. After step down, the primary may be a different
     * server. We must execute serverStatus against the same server to check
     * connection counts. */
-   primary_id = mongoc_topology_select_server_id (client->topology,
-                                                  MONGOC_SS_WRITE,
-                                                  NULL /* read prefs */,
-                                                  NULL /* chosen read mode */,
-                                                  &error);
+   primary_id =
+      mongoc_topology_select_server_id (client->topology,
+                                        MONGOC_SS_WRITE,
+                                        NULL /* read prefs */,
+                                        NULL /* chosen read mode */,
+                                        NULL /* deprioritized servers */,
+                                        &error);
    ASSERT_OR_PRINT (primary_id, error);
    conn_count = _connection_count (client, primary_id);
 
@@ -241,11 +243,13 @@ test_not_primary_keep_pool (mongoc_client_t *client)
    /* Store the primary ID. After step down, the primary may be a different
     * server. We must execute serverStatus against the same server to check
     * connection counts. */
-   primary_id = mongoc_topology_select_server_id (client->topology,
-                                                  MONGOC_SS_WRITE,
-                                                  NULL /* read prefs */,
-                                                  NULL /* chosen read mode */,
-                                                  &error);
+   primary_id =
+      mongoc_topology_select_server_id (client->topology,
+                                        MONGOC_SS_WRITE,
+                                        NULL /* read prefs */,
+                                        NULL /* chosen read mode */,
+                                        NULL /* deprioritized servers */,
+                                        &error);
    ASSERT_OR_PRINT (primary_id, error);
    conn_count = _connection_count (client, primary_id);
    res = mongoc_database_command_simple (
@@ -314,11 +318,13 @@ test_not_primary_reset_pool (mongoc_client_t *client)
    /* Store the primary ID. After step down, the primary may be a different
     * server. We must execute serverStatus against the same server to check
     * connection counts. */
-   primary_id = mongoc_topology_select_server_id (client->topology,
-                                                  MONGOC_SS_WRITE,
-                                                  NULL /* read prefs */,
-                                                  NULL /* chosen read mode */,
-                                                  &error);
+   primary_id =
+      mongoc_topology_select_server_id (client->topology,
+                                        MONGOC_SS_WRITE,
+                                        NULL /* read prefs */,
+                                        NULL /* chosen read mode */,
+                                        NULL /* deprioritized servers */,
+                                        &error);
    ASSERT_OR_PRINT (primary_id, error);
    conn_count = _connection_count (client, primary_id);
    res = mongoc_database_command_simple (
@@ -391,11 +397,13 @@ test_shutdown_reset_pool (mongoc_client_t *client)
    /* Store the primary ID. After step down, the primary may be a different
     * server. We must execute serverStatus against the same server to check
     * connection counts. */
-   primary_id = mongoc_topology_select_server_id (client->topology,
-                                                  MONGOC_SS_WRITE,
-                                                  NULL /* read prefs */,
-                                                  NULL /* chosen read mode */,
-                                                  &error);
+   primary_id =
+      mongoc_topology_select_server_id (client->topology,
+                                        MONGOC_SS_WRITE,
+                                        NULL /* read prefs */,
+                                        NULL /* chosen read mode */,
+                                        NULL /* deprioritized servers */,
+                                        &error);
    ASSERT_OR_PRINT (primary_id, error);
    conn_count = _connection_count (client, primary_id);
    res = mongoc_database_command_simple (
@@ -462,11 +470,13 @@ test_interrupted_shutdown_reset_pool (mongoc_client_t *client)
    /* Store the primary ID. After step down, the primary may be a different
     * server. We must execute serverStatus against the same server to check
     * connection counts. */
-   primary_id = mongoc_topology_select_server_id (client->topology,
-                                                  MONGOC_SS_WRITE,
-                                                  NULL /* read prefs */,
-                                                  NULL /* chosen read mode */,
-                                                  &error);
+   primary_id =
+      mongoc_topology_select_server_id (client->topology,
+                                        MONGOC_SS_WRITE,
+                                        NULL /* read prefs */,
+                                        NULL /* chosen read mode */,
+                                        NULL /* deprioritized servers */,
+                                        &error);
    ASSERT_OR_PRINT (primary_id, error);
    conn_count = _connection_count (client, primary_id);
    res = mongoc_database_command_simple (
