@@ -13,6 +13,9 @@ cannot_resolve (const mongoc_uri_t *uri,
                 void *user_data,
                 bson_error_t *error)
 {
+   BSON_UNUSED (uri);
+   BSON_UNUSED (user_data);
+
    bson_set_error (error,
                    MONGOC_ERROR_STREAM,
                    MONGOC_ERROR_STREAM_NAME_RESOLUTION,
@@ -88,9 +91,9 @@ static void
 test_server_selection_error_dns_direct_single (void)
 {
    server_selection_error_dns (
-      "mongodb://example-localhost:27017/",
+      "mongodb://example-localhost.invalid:27017/",
       "No suitable servers found (`serverSelectionTryOnce` set): "
-      "[Fake error for 'example-localhost']",
+      "[Fake error for 'example-localhost.invalid']",
       false,
       false);
 }
@@ -98,10 +101,12 @@ test_server_selection_error_dns_direct_single (void)
 static void
 test_server_selection_error_dns_direct_pooled (void *ctx)
 {
+   BSON_UNUSED (ctx);
+
    server_selection_error_dns (
-      "mongodb://example-localhost:27017/",
+      "mongodb://example-localhost.invalid:27017/",
       "No suitable servers found: `serverSelectionTimeoutMS` expired: "
-      "[Fake error for 'example-localhost']",
+      "[Fake error for 'example-localhost.invalid']",
       false,
       true);
 }
@@ -110,10 +115,11 @@ static void
 test_server_selection_error_dns_multi_fail_single (void)
 {
    server_selection_error_dns (
-      "mongodb://example-localhost:27017,other-example-localhost:27017/",
+      "mongodb://"
+      "example-localhost.invalid:27017,other-example-localhost.invalid:27017/",
       "No suitable servers found (`serverSelectionTryOnce` set):"
-      " [Fake error for 'example-localhost']"
-      " [Fake error for 'other-example-localhost']",
+      " [Fake error for 'example-localhost.invalid']"
+      " [Fake error for 'other-example-localhost.invalid']",
       false,
       false);
 }
@@ -121,11 +127,14 @@ test_server_selection_error_dns_multi_fail_single (void)
 static void
 test_server_selection_error_dns_multi_fail_pooled (void *ctx)
 {
+   BSON_UNUSED (ctx);
+
    server_selection_error_dns (
-      "mongodb://example-localhost:27017,other-example-localhost:27017/",
+      "mongodb://"
+      "example-localhost.invalid:27017,other-example-localhost.invalid:27017/",
       "No suitable servers found: `serverSelectionTimeoutMS` expired:"
-      " [Fake error for 'example-localhost']"
-      " [Fake error for 'other-example-localhost']",
+      " [Fake error for 'example-localhost.invalid']"
+      " [Fake error for 'other-example-localhost.invalid']",
       false,
       true);
 }
@@ -137,9 +146,9 @@ _test_server_selection_error_dns_multi_success (bool pooled)
    char *uri_str;
 
    host = test_framework_get_host ();
-   uri_str = bson_strdup_printf ("mongodb://example-localhost:27017,"
+   uri_str = bson_strdup_printf ("mongodb://example-localhost.invalid:27017,"
                                  "%s:%d,"
-                                 "other-example-localhost:27017/",
+                                 "other-example-localhost.invalid:27017/",
                                  host,
                                  test_framework_get_port ());
 
@@ -152,12 +161,16 @@ _test_server_selection_error_dns_multi_success (bool pooled)
 static void
 test_server_selection_error_dns_multi_success_single (void *context)
 {
+   BSON_UNUSED (context);
+
    _test_server_selection_error_dns_multi_success (false);
 }
 
 static void
 test_server_selection_error_dns_multi_success_pooled (void *context)
 {
+   BSON_UNUSED (context);
+
    _test_server_selection_error_dns_multi_success (true);
 }
 
@@ -215,12 +228,16 @@ _test_server_selection_uds_auth_failure (bool pooled)
 static void
 test_server_selection_uds_auth_failure_single (void *context)
 {
+   BSON_UNUSED (context);
+
    _test_server_selection_uds_auth_failure (false);
 }
 
 static void
 test_server_selection_uds_auth_failure_pooled (void *context)
 {
+   BSON_UNUSED (context);
+
    _test_server_selection_uds_auth_failure (true);
 }
 
@@ -271,12 +288,16 @@ _test_server_selection_uds_not_found (bool pooled)
 static void
 test_server_selection_uds_not_found_single (void *context)
 {
+   BSON_UNUSED (context);
+
    _test_server_selection_uds_not_found (false);
 }
 
 static void
 test_server_selection_uds_not_found_pooled (void *context)
 {
+   BSON_UNUSED (context);
+
    _test_server_selection_uds_not_found (true);
 }
 
