@@ -14,13 +14,17 @@ with_transaction_fail_transient_txn (mongoc_client_session_t *session,
                                      bson_t **reply,
                                      bson_error_t *error)
 {
-   bson_t labels;
+   bson_array_builder_t *labels;
+
+   BSON_UNUSED (ctx);
+   BSON_UNUSED (error);
 
    _mongoc_usleep (session->with_txn_timeout_ms * 1000);
 
    *reply = bson_new ();
-   BSON_APPEND_ARRAY_BEGIN (*reply, "errorLabels", &labels);
-   BSON_APPEND_UTF8 (&labels, "0", TRANSIENT_TXN_ERR);
+   BSON_APPEND_ARRAY_BUILDER_BEGIN (*reply, "errorLabels", &labels);
+   bson_array_builder_append_utf8 (labels, TRANSIENT_TXN_ERR, -1);
+   bson_append_array_builder_end (*reply, labels);
 
    return false;
 }
@@ -31,6 +35,10 @@ with_transaction_do_nothing (mongoc_client_session_t *session,
                              bson_t **reply,
                              bson_error_t *error)
 {
+   BSON_UNUSED (session);
+   BSON_UNUSED (ctx);
+   BSON_UNUSED (reply);
+   BSON_UNUSED (error);
    return true;
 }
 
@@ -42,6 +50,8 @@ test_with_transaction_timeout (void *ctx)
    bson_error_t error;
    bool res;
 
+   BSON_UNUSED (ctx);
+
    client = test_framework_new_default_client ();
 
    session = mongoc_client_start_session (client, NULL, &error);
